#include "config.h"
#include "libwtref.h"
#include <stddef.h>
#include <stdlib.h>
#include "blake2/blake2.h"

static int ha_blake2_init(const struct wtr_hashalg* const alg,
	void** const ctx)
{
	blake2b_state* const blake_ctx =
		(blake2b_state*)malloc(sizeof(blake2b_state));

	if(NULL==blake_ctx)
	{
		return 1;
	}

	blake2b_init(blake_ctx, alg->digest_len);

	*ctx = blake_ctx;

	return 0;
}

static void ha_blake2_update(const struct wtr_hashalg* const alg,
	void* const ctx, const void* data, size_t size)
{
	blake2b_update((blake2b_state*)ctx, data, size);
}

static void ha_blake2_final(const struct wtr_hashalg* const alg,
	void* const ctx, void* digest)
{
	blake2b_final((blake2b_state*)ctx, digest, alg->digest_len);
	free(ctx);
}

const struct wtr_hashalg libwtref_hash_algs[] =
{
	{
		"BLAKE2-256",
		WTBHA_BLAKE2_256,
		256/8,
		ha_blake2_init,
		ha_blake2_update,
		ha_blake2_final,
	},
	{
		"BLAKE2-384",
		WTBHA_BLAKE2_384,
		384/8,
		ha_blake2_init,
		ha_blake2_update,
		ha_blake2_final,
	},
	{
		"BLAKE2-512",
		WTBHA_BLAKE2_512,
		512/8,
		ha_blake2_init,
		ha_blake2_update,
		ha_blake2_final,
	},
// TODO
/*	{ "SHA2-256",   WTBHA_SHA2_256,   256/8 }, */
/*	{ "SHA2-384",   WTBHA_SHA2_384,   384/8 }, */
/*	{ "SHA2-512",   WTBHA_SHA2_512,   512/8 }, */
	{ 0 }
};
