#include "libwtref.h"
#include <stdlib.h>
#include <string.h>
#include <libvmma/vmmu32.h>

static unsigned char so_acquire(struct vmmu32_segment* const segment)
{
	if(NULL==(segment->u.direct.base=(vmmu_uint8_t*)malloc(segment->size)))
	{
		return 1;
	}

	return 0;
}

static unsigned char so_release(struct vmmu32_segment* const segment)
{
	free(segment->u.direct.base);
	return 0;
}

static const struct vmmu32_segops segops =
{
	so_acquire,
	so_release,
};

int libwtref_segments_create(struct libwtref_segments* const segs,
	wt_u32 heap_size, wt_u32 gran_hint)
{
	/* TODO: improve segmentation */

	const wt_u32 szsh = 20; /* 1 MiB */
	const wt_u32 size = ((wt_u32)1) << szsh;

	const wt_u32 nsegs = heap_size >> szsh;

	wt_u32 i;

	if(NULL==(segs->vma_segs=(struct vmmu32_segment*)malloc(
		sizeof(struct vmmu32_segment)*(1+nsegs))))
	{
		return 1;
	}

	memset(segs->vma_segs, 0, sizeof(struct vmmu32_segment)*(1+nsegs));

	segs->vma_segs[0].type = VMMU_SEGTYPE_EMPTY;

	for(i=1;i<=nsegs;++i)
	{
		segs->vma_segs[i].type   = VMMU_SEGTYPE_DIRECT;
		segs->vma_segs[i].pflags = VMMU_SEGPFL_READ | VMMU_SEGPFL_WRITE;
		segs->vma_segs[i].size   = size;
		segs->vma_segs[i].segops = &segops;
	}

	segs->vma_nsegs  = 1 + nsegs;
	segs->vma_szsh   = szsh;
	segs->heap_start = size;
	segs->heap_size  = nsegs * size;
	segs->heap_gran  = 64;

	return 0;
}

void libwtref_segments_destroy(struct libwtref_segments* const segs)
{
	free(segs->vma_segs);
}
