/* $Id: ast_exceptioncompound.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_EXCEPTIONCOMPOUND_H
#define WTPARSER_AST_EXCEPTIONCOMPOUND_H

#include "../astnode.h"
#include <deque>

namespace wtast
{

class Statement;
class EHandlers;
class Finally;

class ExceptionCompound : public AstNode
{
	typedef std::deque<Statement*> statements_t;
	statements_t mStatements;
	EHandlers*   mEHandlers;
	Finally*     mFinally;

public:
	template<typename Iter>
	ExceptionCompound(const Iter & first, const Iter & last, EHandlers* eh, Finally* fin)
		: AstNode()
		, mStatements(first, last)
		, mEHandlers(eh)
		, mFinally(fin)
	{
		// ok
	}

	~ExceptionCompound();

	void Accept(AstVisitor* vis);

	ExceptionCompound* Copy();

	Kind GetKind() const
	{
		return EXCEPTIONCOMPOUND;
	}

	std::size_t GetNumberOfStatements() const
	{
		return mStatements.size();
	}

	Statement* GetStatement(std::size_t idx)
	{
		if(idx<mStatements.size())
		{
			return mStatements[idx];
		}
		else
		{
			return 0;
		}
	}

	EHandlers* GetEHandlers()
	{
		return mEHandlers;
	}

	Finally* GetFinally()
	{
		return mFinally;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_EXCEPTIONCOMPOUND_H

