/* $Id: ast_functionpointercall.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_FUNCTIONPOINTERCALL_H
#define WTPARSER_AST_FUNCTIONPOINTERCALL_H

#include "../astnode.h"
#include "ast_expression.h"
#include "ast_typeinfo.h"
#include <string>

namespace wtast
{

class FunctionType;
class FunctionSignature;
class Expression;

class FunctionPointerCall : public Expression
{
	FunctionType* mFuncType;
	Expression*   mFuncPtrAccess;
	Expression*   mArgumentExpression;
	TypeInfo*     mTypeInfo;
	bool          mIsTypeInfoAllocated;

public:
	FunctionPointerCall(FunctionType* functype, Expression* funcptraccess, Expression* argexpr);
	~FunctionPointerCall();

	void Accept(AstVisitor* vis);

	FunctionPointerCall* Copy();

	Kind GetKind() const
	{
		return FUNCTIONPOINTERCALL;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		if(mFuncPtrAccess->OriginateFrom(varclass))
		{
			return true;
		}

		if(mArgumentExpression && mArgumentExpression->OriginateFrom(varclass))
		{
			return true;
		}

		return Variable::READONLY==varclass || Variable::LOCAL==varclass;
	}

	TypeInfo* GetType()
	{
		return mTypeInfo;
	}

	FunctionType* GetFunctionType()
	{
		return mFuncType;
	}

	FunctionSignature* GetFunctionSignature();

	Expression* GetFunctionPointerAccess()
	{
		return mFuncPtrAccess;
	}

	Expression* GetArgumentExpression()
	{
		return mArgumentExpression;
	}

	virtual bool IsTemporary() const { return true; } // from wtast::Expression
};

} // namespace wtast

#endif // WTPARSER_AST_FUNCTIONPOINTERCALL_H
