/* $Id: ast_numberliteral.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_NUMBERLITERAL_H
#define WTPARSER_AST_NUMBERLITERAL_H

#include "../astnode.h"
#include "ast_expression.h"
#include "welltype/wtruntime.h"
#include <string>

namespace wtast
{

class Type;

class NumberLiteral : public Expression
{
	Type* mType;
	bool  mIsInteger;
	bool  mIsSigned;
	bool  mIs64Bit;

	union
	{
	wt_i32 mI32;
	wt_u32 mU32;
	wt_i64 mI64;
	wt_u64 mU64;
	wt_f32 mF32;
	wt_f64 mF64;
	};

	NumberLiteral(Type* type, bool isint, bool issig, bool is64, wt_u64 data);

public:
	NumberLiteral(Type* type, wt_i32 literal) : Expression(), mType(type), mIsInteger(true), mIsSigned(true), mIs64Bit(false), mI32(literal) { }
	NumberLiteral(Type* type, wt_u32 literal) : Expression(), mType(type), mIsInteger(true), mIsSigned(false), mIs64Bit(false), mU32(literal) { }
	NumberLiteral(Type* type, wt_i64 literal) : Expression(), mType(type), mIsInteger(true), mIsSigned(true), mIs64Bit(true), mI64(literal) { }
	NumberLiteral(Type* type, wt_u64 literal) : Expression(), mType(type), mIsInteger(true), mIsSigned(false), mIs64Bit(true), mU64(literal) { }
	NumberLiteral(Type* type, wt_f32 literal) : Expression(), mType(type), mIsInteger(false), mIsSigned(true), mIs64Bit(false), mF32(literal) { }
	NumberLiteral(Type* type, wt_f64 literal) : Expression(), mType(type), mIsInteger(false), mIsSigned(true), mIs64Bit(true), mF64(literal) { }

	~NumberLiteral();

	void Accept(AstVisitor* vis);

	NumberLiteral* Copy();

	Kind GetKind() const
	{
		return NUMBERLITERAL;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		return Variable::READONLY==varclass || Variable::LOCAL==varclass;
	}

	const void* GetData() const { return &mU64; }
	wt_i32 GetLiteralI32() { return mI32; }
	wt_u32 GetLiteralU32() { return mU32; }
	wt_i64 GetLiteralI64() { return mI64; }
	wt_u64 GetLiteralU64() { return mU64; }
	wt_f32 GetLiteralF32() { return mF32; }
	wt_f64 GetLiteralF64() { return mF64; }

	bool IsInteger() const
	{
		return mIsInteger;
	}

	bool IsSigned() const
	{
		return mIsSigned;
	}

	bool Is64Bit() const
	{
		return mIs64Bit;
	}

	std::string ToString() const;

	TypeInfo* GetType();

	virtual bool IsLiteral() const { return true; } // from wtast::Expression
};

} // namespace wtast

#endif // WTPARSER_AST_NUMBERLITERAL_H
