/* $Id: ast_postcondition.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_POSTCONDITION_H
#define WTPARSER_AST_POSTCONDITION_H

#include <deque>
#include "../astnode.h"
#include "welltype/wtruntime.h"

namespace wtast
{

class Expression;
class VariableDeclaration;

class PostCondition : public AstNode
{
	typedef std::deque<VariableDeclaration*> vardecls_t;

	vardecls_t  mReturnValues;
	vardecls_t  mOldValues;
	Expression* mCondition;

public:
	PostCondition(Expression* cond)
		: AstNode()
		, mCondition(cond)
	{
		// ok
	}

	~PostCondition();

	void AddReturnValue(wtast::VariableDeclaration* ret)
	{
		mReturnValues.push_back(ret);
	}

	void AddOldValue(wtast::VariableDeclaration* old)
	{
		mOldValues.push_back(old);
	}

	template<typename Iter>
	void AddReturnValues(Iter begin, Iter end)
	{
		mReturnValues.insert(mReturnValues.end(), begin, end);
	}

	template<typename Iter>
	void AddOldValues(Iter begin, Iter end)
	{
		mOldValues.insert(mOldValues.end(), begin, end);
	}

	std::size_t GetNumReturnValues() const
	{
		return mReturnValues.size();
	}

	VariableDeclaration* GetReturnValue(std::size_t idx)
	{
		if(idx<mReturnValues.size())
		{
			return mReturnValues[idx];
		}
		else
		{
			return 0;
		}
	}

	std::size_t GetNumOldValues() const
	{
		return mOldValues.size();
	}

	VariableDeclaration* GetOldValue(std::size_t idx)
	{
		if(idx<mOldValues.size())
		{
			return mOldValues[idx];
		}
		else
		{
			return 0;
		}
	}

	void Accept(AstVisitor* vis);

	PostCondition* Copy();

	Kind GetKind() const
	{
		return POSTCONDITION;
	}

	Expression* GetCondition()
	{
		return mCondition;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_POSTCONDITION_H
