/* $Id: ast_stringliteral.h 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_STRINGLITERAL_H
#define WTPARSER_AST_STRINGLITERAL_H

#include "../astnode.h"
#include "ast_expression.h"
#include "welltype/wtruntime.h"
#include <string>

namespace wtast
{

class Type;

class StringLiteral : public Expression
{
	Type*       mType;
	std::string mLiteral;

public:
	StringLiteral(Type* type, const std::string & literal)
		: Expression()
		, mType(type)
		, mLiteral(literal)
	{
		// ok
	}

	~StringLiteral();

	void Accept(AstVisitor* vis);

	StringLiteral* Copy();

	Kind GetKind() const
	{
		return STRINGLITERAL;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		return Variable::READONLY==varclass || Variable::LOCAL==varclass;
	}

	std::string & GetLiteral()
	{
		return mLiteral;
	}

	TypeInfo* GetType();

	virtual bool IsLiteral() const { return true; } // from wtast::Expression
	virtual bool IsTemporary() const { return true; } // from wtast::Expression
};

} // namespace wtast

#endif // WTPARSER_AST_STRINGLITERAL_H
