/* $Id: ast_switchstatement.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_switchstatement.h"
#include "ast_switchcase.h"
#include "ast_expression.h"
#include "../astvisitor.h"

wtast::SwitchStatement::~SwitchStatement()
{
	delete mCondition;

	while(!mCases.empty())
	{
		delete mCases.back();
		mCases.pop_back();
	}
}

void wtast::SwitchStatement::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		mCondition->Accept(vis);

		for(cases_t::iterator it=mCases.begin(),end=mCases.end();it!=end;++it)
		{
			(*it)->Accept(vis);
		}

		vis->EndVisit(this);
	}
}

wtast::SwitchStatement* wtast::SwitchStatement::Copy()
{
	SwitchStatement* swstmt = new SwitchStatement(mCondition->Copy());

	for(cases_t::iterator it=mCases.begin(),end=mCases.end();it!=end;++it)
	{
		swstmt->AddCase((*it)->Copy());
	}

	return swstmt;
}
