//================================================================================================
// $Id: lex.yy.cc.cpp 4611 2015-11-19 16:42:49Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		lex.yy.cc extension
// Authors:
//		Aron Barath, 2014-2015
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#include "lex.yy.cc.h"
#include "problemhandler.h"
#include "ParserHelper.h"

WTFlexLexer::WTFlexLexer(wtparser::ParserHelper* h, wtparser::IFile* file, WTFlexLexer* prev)
	: yyFlexLexer(0, 0)
	, mFile(file)
	, mProbHan(h->GetProblemHandler())
	, mPrevious(prev)
	, mHelper(h)
	, mColumn(1)
{
	// ok
}

void WTFlexLexer::LexerOutput(const char* /*buf*/, int /*size*/)
{
	// ignore
}

void WTFlexLexer::LexerError(const char* msg)
{
	mProbHan.AddError(mHelper->GetFilename(), lineno(), std::string("Lexical error: ") + msg);
}

void WTFlexLexer_UpdatePositionInfo(WTFlexLexer* lexer, unsigned int len)
{
	lexer->mStartPosInfo.line = lexer->mEndPosInfo.line = lexer->lineno();
	lexer->mStartPosInfo.col  = lexer->mColumn;
	lexer->mColumn           += len;
	lexer->mEndPosInfo.col    = lexer->mColumn - 1;
}

void WTFlexLexer_NewLine(WTFlexLexer* lexer)
{
	lexer->mColumn = 1;
}

void WTFlexLexer_WhiteSpaces(WTFlexLexer* lexer, const char* str, unsigned int len)
{
	for(;len>0;++str,--len)
	{
		if('\n'==*str)
		{
			lexer->mColumn = 1;
			return;
		}
	}
}
