//================================================================================================
// $Id: symtab.cpp 6309 2023-01-16 20:19:42Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		Symbol table
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#include "symtab.h"
#include "ParserHelper.h"
#include "ast/ast_type.h"

const std::string wtparser::SymTab::UnnamedFrame("");

wtparser::SymTab::SymTab(wtparser::ParserHelper* helper)
	: mHelper(helper)
	, mGlobal(wtparser::ParserHelper::GlobalBlockName)
{
	// ok
}

bool wtparser::SymTab::EnterBlock()
{
	return EnterBlock(UnnamedFrame);
}

bool wtparser::SymTab::EnterBlock(const std::string & name)
{
	if(!name.empty())
	{
		// ha a frame-nek van neve, akkor egyedinek kell lennie

		for(framestack_t::iterator it=mFrameStack.begin(),end=mFrameStack.end();it!=end;++it)
		{
			if(name == it->name)
			{
				// ilyen nevu frame mar volt

				return false;
			}
		}
	}

	if(!mFrameStack.empty())
	{
		mFrameStack.push_back(Frame(name, mFrameStack.back()));
	}
	else
	{
		mFrameStack.push_back(Frame(name));
	}

	return true;
}

void wtparser::SymTab::LeaveBlock()
{
//	Frame & frame = mFrameStack.back();

	mFrameStack.pop_back();
}

bool wtparser::SymTab::AddVariable(wtast::Type* type, const std::string & name, UniqueId unique_id, bool tagged, wtast::AstNode::Variable::Class varclass)
{
	Frame & frame = GetInsertionFrame(varclass);

	wt_u32 loadsize = type->GetLoadSize();
	Variable* var = frame.Find(name);

	if(var && var->depth==CurrentDepth())
	{
		// egy frame-en belul nem lehet ket ugyanolyan nevu valtozo
		return false;
	}

	frame.Insert(name, type, unique_id, tagged, CurrentDepth(), loadsize, varclass);

	return true;
}

void wtparser::SymTab::HideVariable(UniqueId unique_id)
{
	for(framestack_t::iterator it=mFrameStack.begin(),end=mFrameStack.end();it!=end;++it)
	{
		it->HideByUniqueId(unique_id);
	}

	mGlobal.HideByUniqueId(unique_id);
}

bool wtparser::SymTab::Resolve(Resolver & resolver, const std::string & var_name, const std::string & frame_name)
{
	Variable* var   = 0;
	Frame*    frame = 0;

	if(frame_name!=wtparser::ParserHelper::GlobalBlockName)
	{
		if(frame_name.empty())
		{
			if(!mFrameStack.empty())
			{
				frame = &mFrameStack.back();
			}
			else
			{
				frame = &mGlobal;
			}
		}
		else
		{
			for(framestack_t::reverse_iterator it=mFrameStack.rbegin(),end=mFrameStack.rend();it!=end;++it)
			{
				if(frame_name==it->name)
				{
					frame = &*it;
					break;
				}
			}
		}
	}

	if(&mGlobal!=frame && (!frame || frame->Find(var_name)==0))
	{
		frame = &mGlobal;
	}

	if((var=frame->Find(var_name))!=0)
	{
		resolver.type       = var->type;
		resolver.name       = var->name;
		resolver.frame_name = frame->name;
		resolver.unique_id  = var->unique_id;
		resolver.tagged     = var->tagged;
		resolver.varclass   = var->varclass;

		return true;
	}

	return false;
}

void wtparser::SymTab::Frame::Insert(const std::string & name, wtast::Type* type, UniqueId unique_id, bool tagged, wt_u32 depth, wt_u32 loadsize, wtast::AstNode::Variable::Class varclass)
{
	vars[name] = Variable(name, type, unique_id, tagged, depth, varclass);
}

bool wtparser::SymTab::Frame::HideByUniqueId(UniqueId unique_id)
{
	for(varmap_t::iterator it=vars.begin(),end=vars.end();it!=end;++it)
	{
		if(it->second.unique_id==unique_id)
		{
			vars.erase(it);
			return true;
		}
	}

	return false;
}
